#!/bin/bash
mkdir build
cd build
cmake -DREAKTORO_BUILD_ALL=ON \
      -DREAKTORO_PYTHON_INSTALL_PREFIX=$PREFIX \
      -DREAKTORO_THIRDPARTY_EXTRA_INSTALL_ARGS="-DCMAKE_VERBOSE_MAKEFILE:BOOL=ON" \
      -DCMAKE_BUILD_TYPE=Release \
      -DCMAKE_INSTALL_PREFIX=$PREFIX \
      -DCMAKE_INSTALL_LIBDIR=lib \
      -DCMAKE_INCLUDE_PATH=$PREFIX/include \
      -DCMAKE_VERBOSE_MAKEFILE:BOOL=ON \
      -DBOOST_INCLUDE_DIR=$PREFIX/include \
      ..
make VERBOSE=1 -j${CPU_COUNT}
make install
